"""This package provides geometric abstractions for action planning algorithms.

Warning:
    This module docstring is not included in the *Sphinx* documentation.
"""

import typing  # Will be removed from the namespace after being used below


from .geospatial import Direction
from .geospatial import Geospatial
from .geospatial import LocationType

from .location import CartesianLocation
from .location import PolarLocation

from .polygon import CartesianPolygon
from .polygon import PolarPolygon

from .route import CartesianRoute
from .route import PolarRoute

# provide useful aliases

#: Any of :class:`pyrate.plan.geometry.PolarLocation`, :class:`pyrate.plan.geometry.PolarRoute` and
#: :class:`pyrate.plan.geometry.PolarPolygon`.
PolarGeometry = typing.Union[PolarLocation, PolarRoute, PolarPolygon]

#: Any of :class:`pyrate.plan.geometry.CartesianLocation`, :class:`pyrate.plan.geometry.CartesianRoute` and
#: :class:`pyrate.plan.geometry.CartesianPolygon`.
CartesianGeometry = typing.Union[CartesianLocation, CartesianRoute, CartesianPolygon]

del typing

# don't expose .helpers here as it will rarely be used directly

__all__ = [
    "CartesianLocation",
    "CartesianPolygon",
    "CartesianRoute",
    "CartesianGeometry",
    "Direction",
    "Geospatial",
    "LocationType",
    "PolarLocation",
    "PolarPolygon",
    "PolarRoute",
    "PolarGeometry",
]
