"""
The ``graph`` module provides navigation tools where the world is modeled as a graph.
This includes generating a graph, assigning properties to nodes of the graph and finding good paths on it.

Two graph models are provided:
:class:`~pyrate.plan.graph.graph.NavigationGraph` is a generic implementation and in
:class:`~pyrate.plan.graph.geo_graph.GeoNavigationGraph`, nodes are referenced to geographical locations.
"""

from .graph import NavigationGraph

from .geo_graph import GeoNavigationGraph

from .generate import angular_distance_for
from .generate import create_earth_graph
from .generate import great_circle_distance_distance_for
from .generate import min_required_frequency

__all__ = [
    "GeoNavigationGraph",
    "NavigationGraph",
    "angular_distance_for",
    "create_earth_graph",
    "great_circle_distance_distance_for",
    "min_required_frequency",
]
