"""Provides the means to describe possible routes (via TimingFrames) in a given environment (CostModel)
and find a possible route (via Optimizer) in a given context.

:class:`~near_planner.planning_backend.timing_frame.TimingFrame` encapsulates a
:class:`~pyrate.plan.geometry.route.CartesianRoute` instance. The class further provides an option to
simulate the route with the help of a :class:`~near_planner.planning_backend.polar_model.PolarModel`
instance, simulating timing and speed information. This is done via the method
:meth:`~near_planner.planning_backend.timing_frame.TimingFrame.update_times`. Using a
:class:`~near_planner.planning_backend.cost_model.CostModel` a
:class:`~near_planner.planning_backend.timing_frame.TimingFrame` can be evaluated to an
:class:`~near_planner.planning_backend.timing_frame.EvaluatedTimingFrame`

The subclass :class:`~near_planner.planning_backend.timing_frame.EvaluatedTimingFrame` allows for further
saving of information on possible collisions and distances to
:class:`~near_planner.planning_backend.obstacle.Obstacle`\\ s . These are conveniently provided via
properties. If the information is deemed incorrect, the frame can be revoked via
:meth:`~near_planner.planning_backend.timing_frame.EvaluatedTimingFrame.revoke` and turned back into a
:class:`~near_planner.planning_backend.timing_frame.TimingFrame`.

The :class:`~near_planner.planning_backend.cost_model.CostModel` class allows to conveniently store
obstacles and environment information. A :class:`~near_planner.planning_backend.timing_frame.TimingFrame`
can be turned into an :class:`~near_planner.planning_backend.timing_frame.EvaluatedTimingFrame` via the
:meth:`~near_planner.planning_backend.cost_model.CostModel.evaluate` method. This method also calculates
the cost of taking the TimingFrame by various metrics, stores it in
:attr:`~near_planner.planning_backend.timing_frame.EvaluatedTimingFrame.actual_cost`. The calculated cost is
further returned.

The :class:`~near_planner.planning_backend.optimizer.Optimizer` class allows to determine, hopefully
optimal, routes towards a goal, given by a :class:`pyrate.plan.geometry.location.CartesianLocation`. This
is done by initializing the :class:`~near_planner.planning_backend.optimizer.Optimizer` with information
about the current environment. After that a goal and
:class:`~near_planner.planning_backend.holders.OptimizationParameters` can be provided to the method
:meth:`~near_planner.planning_backend.optimizer.Optimizer.optimizer` which generates a hopefully optimal
route candidate. The :class:`~near_planner.planning_backend.optimizer.Optimizer` is furthermore able to
react to a changing :class:`~near_planner.planning_backend.obstacle.Obstacle` population by changing its
state via several methods.

The package further provides various (data)classes and methods to help in the interaction with the core
classes. This includes several cost functions to provide
:class:`~near_planner.planning_backend.obstacle.Obstacle` with, functions to merge numpy arrays and
dataclasses to wrap various parameters in.
"""
